//=============================================================================
// LazyParser.NET - C# Expression Parser for .NET 2.0 
//
// Copyright (c) 2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;

namespace Activa.LazyParser
{
    internal class BinaryExpressionHelper
    {
        public static object CalcUInt32_Int32(string op, object p1, object p2)
        {
            uint? v1 = (uint?)p1;
            int? v2 = (int?)p2;

            switch (op)
            {
                case ">>": return v1 >> v2;
                case "<<": return v1 << v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcInt64_Int32(string op, object p1, object p2)
        {
            long? v1 = (long?)p1;
            int? v2 = (int?)p2;

            switch (op)
            {
                case ">>": return v1 >> v2;
                case "<<": return v1 << v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcUInt64_Int32(string op, object p1, object p2)
        {
            ulong? v1 = (ulong?)p1;
            int? v2 = (int?)p2;

            switch (op)
            {
                case ">>": return v1 >> v2;
                case "<<": return v1 << v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcBool(string op, object p1, object p2)
        {
            bool? v1 = (bool?)p1;
            bool? v2 = (bool?)p2;

            switch (op)
            {
                case "^": return v1 ^ v2;
                case "&": return v1 & v2;
                case "|": return v1 | v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcInt32(string op, object p1,object p2)
        {
            int? v1 = (int?) p1;
            int? v2 = (int?) p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "^": return v1 ^ v2;
                case "&": return v1 & v2;
                case "|": return v1 | v2;
                case ">>": return v1 >> v2;
                case "<<": return v1 << v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcUInt32(string op, object p1, object p2)
        {
            uint? v1 = (uint?)p1;
            uint? v2 = (uint?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "^": return v1 ^ v2;
                case "&": return v1 & v2;
                case "|": return v1 | v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcInt64(string op, object p1, object p2)
        {
            long? v1 = (long?)p1;
            long? v2 = (long?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "^": return v1 ^ v2;
                case "&": return v1 & v2;
                case "|": return v1 | v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcUInt64(string op, object p1, object p2)
        {
            ulong? v1 = (ulong?)p1;
            ulong? v2 = (ulong?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "^": return v1 ^ v2;
                case "&": return v1 & v2;
                case "|": return v1 | v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcFloat(string op, object p1, object p2)
        {
            float? v1 = (float?)p1;
            float? v2 = (float?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcDouble(string op, object p1, object p2)
        {
            double? v1 = (double?)p1;
            double? v2 = (double?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcDecimal(string op, object p1, object p2)
        {
            decimal? v1 = (decimal?)p1;
            decimal? v2 = (decimal?)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "-": return v1 - v2;
                case "/": return v1 / v2;
                case "*": return v1 * v2;
                case "%": return v1 % v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
                case "<=": return v1 <= v2;
                case ">=": return v1 >= v2;
                case "<": return v1 < v2;
                case ">": return v1 > v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcString(string op, object p1, object p2)
        {
            string v1 = (string)p1;
            string v2 = (string)p2;

            switch (op)
            {
                case "+": return v1 + v2;
                case "==": return v1 == v2;
                case "!=": return v1 != v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcStringObject(string op, object p1, object p2)
        {
            string v1 = (string)p1;
            object v2 = p2;

            switch (op)
            {
                case "+": return v1 + v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcObjectString(string op, object p1, object p2)
        {
            object v1 = p1;
            string v2 = (string) p2;

            switch (op)
            {
                case "+": return v1 + v2;
            }

            throw new ArithmeticException();
        }

        public static object CalcObject(string op, object p1, object p2)
        {
            if (op != "==" && op != "!=")
                throw new ArithmeticException();

            bool isEqual;

            if (p1 == null || p2 == null)
                isEqual = object.ReferenceEquals(p1,p2);
            else
                isEqual = p1.Equals(p2);

            return isEqual == (op == "==");
        }


    }
}